
import java.nio.ByteBuffer;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;
import java.nio.channels.Channels;
import java.io.IOException;

/**
 * Copy from channel to channel with simple iterating loop
 * @author Ron Hitchens (ron@ronsoft.com)
 */
public class ChannelCopySimple
{
	public static void main (String [] argv)
		throws IOException
	{
		// Wrap channels around System.in and System.out
		ReadableByteChannel source = Channels.newChannel (System.in);
		WritableByteChannel dest = Channels.newChannel (System.out);

		channelCopy (source, dest);

		source.close();
		dest.close();
	}

	private static void channelCopy (ReadableByteChannel src,
		WritableByteChannel dest)
		throws IOException
	{
		ByteBuffer buffer = ByteBuffer.allocateDirect (16 * 1024);

		while (src.read (buffer) != -1) {
			// prepare the buffer to be drained
			buffer.flip();

			// make sure the buffer was fully drained.
			while (buffer.hasRemaining()) {
				dest.write (buffer);
			}

			// make the buffer empty, ready for filling
			buffer.clear();
		}
	}
}

